# -*- coding: utf-8 -*-
__title__ = "apapi"
__description__ = "Unofficial Anaplan API Python client library"
__url__ = "https://github.com/DLZaan/apapi"
__version__ = "0.2.0"
__author__ = "Milosz Kierepka"
__author_email__ = "milosz.kierepka+work@gmail.com"
__license__ = "Apache 2.0"
__copyright__ = "Copyright 2022 Milosz Kierepka"
